@@ -4,66 +4,54 @@ use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '0.16';
+our $VERSION = '0.17';
+
+my %introduces = ( 5.022 => [qw[
+                                 <<>> \b{} /n unicode7.0 :const
+                                 fileno-dir ()x= hexfloat chr-inf
+                                 empty-slice /x-unicode
+                              ]],
+                   5.020 => [qw[
+                                 :prototype drand48 %slice unicode6.3
+                                 \p{Unicode} utf8-locale
+                              ]],
+                   5.018 => [qw[
+                                 computed-labels
+                              ]],
+                   5.014 => [qw[
+                                 ?^ /r /d /l /u /a auto-deref
+                                 ^GLOBAL_PHASE \o package-block
+                              ]],
+                   5.012 => [qw[
+                                 package-version ... each-array
+                                 keys-array values-array delete-local
+                                 length-undef \N while-readdir
+                              ]],
+                   5.010 => [qw[
+                                 // ?PARNO ?<> ?| quant+ regex-verbs
+                                 \K \R \gN readline() stack-file-test
+                                 recursive-sort /p
+                              ]],
+                 );
+
+my %deprecated = ( 'auto-deref' => 5.024 );
+
+my %introduced = map {
+    my $version = $_;
+    map { $_ => $version } @{ $introduces{$version} }
+} keys %introduces;
 
 
-my %construct = (
-                 '<<>>'            => 5.022,
-                 '\\b{}'           => 5.022,
-                 '/n'              => 5.022,
-                 'unicode7.0'      => 5.022,
-                 ':const'          => 5.022,
-                 'fileno-dir'      => 5.022,
-                 '()x='            => 5.022,
-                 'hexfloat'        => 5.022,
-                 'chr-inf'         => 5.022,
-                 'empty-slice'     => 5.022,
-                 '/x-unicode'      => 5.022,
+sub deprecated {
+    my $construct = shift;
+    return $construct ? $deprecated{$construct} : keys %deprecated
+}
 
-                 ':prototype'      => 5.020,
-                 'drand48'         => 5.020,
-                 '%slice'          => 5.020,
-                 'unicode6.3'      => 5.020,
-                 '\p{Unicode}'     => 5.020,
-                 'utf8-locale'     => 5.020,
 
-                 'computed-labels' => 5.018,
-
-                 '?^'              => 5.014,
-                 '/r'              => 5.014,
-                 '/d'              => 5.014,
-                 '/l'              => 5.014,
-                 '/u'              => 5.014,
-                 '/a'              => 5.014,
-                 'auto-deref'      => 5.014,
-                 '^GLOBAL_PHASE'   => 5.014,
-                 '\o'              => 5.014,
-                 'package-block'   => 5.014,
-
-                 'package-version' => 5.012,
-                 '...'             => 5.012,
-                 'each-array'      => 5.012,
-                 'keys-array'      => 5.012,
-                 'values-array'    => 5.012,
-                 'delete-local'    => 5.012,
-                 'length-undef'    => 5.012,
-                 '\N'              => 5.012,
-                 'while-readdir'   => 5.012,
-
-                 '//'              => 5.010,
-                 '?PARNO'          => 5.010,
-                 '?<>'             => 5.010,
-                 '?|'              => 5.010,
-                 'quant+'          => 5.010,
-                 'regex-verbs'     => 5.010,
-                 '\K'              => 5.010,
-                 '\R'              => 5.010,
-                 '\gN'             => 5.010,
-                 'readline()'      => 5.010,
-                 'stack-file-test' => 5.010,
-                 'recursive-sort'  => 5.010,
-                 '/p'              => 5.010,
-                );
+sub introduced {
+    my $construct = shift;
+    return $construct ? $introduced{$construct} : keys %introduced
+}
 
 
 sub _position {
@@ -73,20 +61,34 @@ sub _position {
 
 sub import {
     shift;
-    my $version = 0;
-    my $constr;
+    my $min_version = 0;
+    my $max_version = 6;
+    my ($constr, $d_constr);
     for (@_) {
-        if (exists $construct{$_}) {
-            ($version, $constr) = ($construct{$_}, $_)
-                if $construct{$_} > $version;
+        if ($introduced{$_}) {
+            ($min_version, $constr) = ($introduced{$_}, $_)
+                if $introduced{$_} > $min_version;
         } else {
             die "Unknown construct `$_' at ", _position(), "\n";
         }
+
+        if ($deprecated{$_}) {
+            ($max_version, $d_constr) = ($deprecated{$_}, $_)
+                if $deprecated{$_} < $max_version;
+        }
     }
-    die 'Empty construct list at ', _position(), "\n" if $version == 0;
-    eval { require $version; 1 }
+    die 'Empty construct list at ', _position(), "\n" if $min_version == 0;
+
+    my $stable = $] =~ /^[0-5]\.[0-9][0-9][02468]/;
+    my $nearest_stable = $stable ? $] : 0.001 + substr $], 0, 5;
+    warn "Faking version $nearest_stable to test deprecations.\n"
+        unless $stable;
+    die "$d_constr deprecated in $max_version at ", _position()
+        if $max_version <= $nearest_stable;
+
+    eval { require $min_version; 1 }
         or die "Unsupported construct $constr at ", _position(),
-               sprintf " (Perl %.3f)\n", $version;
+               sprintf " (Perl %.3f needed)\n", $min_version;
 }
 
 
@@ -96,7 +98,7 @@ Syntax::Construct - Identify which non-feature constructs are used in the code.
 
 =head1 VERSION
 
-Version 0.16
+Version 0.17
 
 =head1 SYNOPSIS
 
@@ -111,16 +113,78 @@ the rest, there is B<Syntax::Construct>.
       ...
   }
 
+There are two subroutines (not exported) which you can use to query
+the lists of constructs programmatically: C<introduced> and
+C<deprecated> (see below).
+
+  my @constructs = Syntax::Construct::introduced();
+  say "$_ was introduced in ",
+      Syntax::Construct::introduced($_) for @constructs;
+
 =head1 DESCRIPTION
 
 This module provides a simple way of specifying syntactic constructs
 that are not implemented via the L<feature> pragma, but are still not
 compatible with older versions of Perl.
 
+It's the programmer's responsibility to track the constructs and list
+them (but see L<Perl::MinimumVersion> on how to extract the
+information from existing code).
+
+Using C<use Syntax::Construct qw( // );> doesn't really change
+anything if you're running Perl 5.10+, but it gives much better error
+messages in older versions:
+
+  Unsupported construct //
+
+instead of
+
+  Search pattern not terminated
+
+Three groups of people can benefit from the module:
+
+=over 4
+
+=item 1.
+
+The authors of the module using L<Syntax::Construct> win, as they have
+all the constructs in one place (i.e. L<Syntax::Construct>'s
+documentation) and they don't waste their time searching through
+perldeltas and other places.
+
+=item 2.
+
+Users of their modules win as they get meaningful error messages
+telling them what Perl version they need to upgrade to.
+
+=item 3.
+
+The programmer they hired to workaround the problem wins as they know
+what constructs to replace in the code to make it run in the ancient
+version.
+
+=back
+
+
 =head1 EXPORT
 
 Nothing. Using B<Syntax::Construct> with no parameters is an error,
-giving it an empty list is a no-op.
+giving it an empty list is a no-op (but you can then access the
+C<introduced> and C<deprecated> subroutines).
+
+=over 4
+
+=item introduced
+
+Without arguments, returns a list of all the supported
+constructs. With an argument, returns the version in which the given
+construct was introduced.
+
+=item deprecated
+
+Same as C<introduced>, but for deprecated constructs.
+
+=back
 
 =head1 RECOGNISED CONSTRUCTS
 
@@ -317,7 +381,7 @@ L<perl5200delta/use locale now works on UTF-8 locales>
 
 L<perldelta/New double diamond operator>
 
-=head3 \\b{}
+=head3 \b{}
 
 L<perldelta/New b boundaries in regular expressions>
 
@@ -357,19 +421,21 @@ L<perldelta/List slices returning empty lists>
 
 L<perldelta/qr foo x now ignores all Unicode pattern white space>
 
-
 =head1 AUTHOR
 
 E. Choroba, C<< <choroba at cpan.org> >>
 
 =head1 BUGS
 
-Please report any bugs or feature requests to C<bug-syntax-construct at rt.cpan.org>, or through
-the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Syntax-Construct>.  I will be notified, and then you'll
-automatically be notified of progress on your bug as I make changes.
+Please report any bugs or feature requests to the GitHub repository,
+see below.
 
-You can also use GitHub, see below.
+=head2 Unstable Perl versions
 
+In development versions of Perl, the deprecation is tested against the
+coming stable version -- e.g., 5.23 forbids all the deprecated
+constructs of 5.24. The behaviour of the module in such circumstances
+might still be, um, unstable.
 
 =head1 SUPPORT
 
@@ -377,7 +443,6 @@ You can find documentation for this module with the perldoc command.
 
     perldoc Syntax::Construct
 
-
 You can also look for information at:
 
 =over 4
@@ -388,9 +453,9 @@ L<https://github.com/choroba/syntactic-construct>
 
 Feel free to report issues and submit pull requests.
 
-=item * RT: CPAN's request tracker (report bugs here)
+=item * MetaCPAN, Open Source Search Engine for CPAN
 
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Syntax-Construct>
+L<https://metacpan.org/pod/Syntax::Construct>
 
 =item * AnnoCPAN: Annotated CPAN documentation
 
@@ -406,13 +471,14 @@ L<http://search.cpan.org/dist/Syntax-Construct/>
 
 =back
 
+=head1 SEE ALSO
 
-=head1 ACKNOWLEDGEMENTS
+L<Perl::MinimumVersion>
 
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2013 - 2014 E. Choroba.
+Copyright 2013 - 2015 E. Choroba.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the the Artistic License (2.0). You may obtain a
